/**
 * Russian translation for bootstrap-datepicker
 */
;(function($){
    $.fn.datepicker.defaults = {
        monthNames: ["Январь", "Февраль", "Март", "Апрель", "Май", "Июнь", "Июль", "Август", "Сентябрь", "Октябрь", "Ноябрь", "Декабрь"]
        , shortDayNames: ["Вск", "Пнд", "Втр", "Срд", "Чтв", "Птн", "Суб"]
        , startOfWeek: 1
    };
}(jQuery));