#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.exceptions
~~~~~~~~~~~~~~~~~~~

GottWall core exceptions

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

class InvalidData(Exception):
    pass

class InvalidParams(Exception):
    pass
