{% extends "base.html" %}


{% block title %}Dashboard{% endblock %}


{% block content %}

<div class="row chart-control">
  <div class="span8 periods">
    <button class="btn" id="chart-add"><i class="icon-plus"></i> Добавить график</button>
    <span style="display:inline-block;width: 60px;text-align: right;">по</span>
    <div class="btn-group selector">
      <button class="btn" data-type="year">годам</button>
      <button class="btn" data-type="month">месяцам</button>
      <!--<button class="btn" data-type="week">неделям</button>-->
      <button class="btn" data-type="day">дням</button>
      <button class="btn" data-type="hour">часам</button>
      <button class="btn" data-type="minute">минутам</button>
    </div>
  </div>

  <div class="span5 intervals">
    <span style="display:inline-block;width: 60px;text-align: right;">Интервал:</span>
    <span style="display:inline-block;width: 20px;text-align: right;">с</span>
    <input data-datepicker="datepicker" data-date-format="yyyy-mm-dd" type="text" name="start-date" class="input_date start-date" id="from-date">
    <span style="display:inline-block;width: 20px;text-align: right;">по</span>
    <input data-datepicker="datepicker" data-date-format="yyyy-mm-dd" type="text" name="end-date" class="input_date end-date" id="to-date">
  </div>
  <div class="redraw pull-right">
    <button class="btn" id="redraw-button">Перерисовать</button>
  </div>
</div>

<div class="charts" id="charts"></div>
{% endblock %}
