#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.log
~~~~~~~~~~~~

Gottwall logging configuration

:copyright: (c) 2013 by GottWall team, see AUTHORS for more details..
:license: BSD, see LICENSE for more details.
:github: http://github.com/GottWall/gottwall
"""
from logging import getLogger

logger = getLogger()
