.main {
    padding: 60px 0;
}

.bs-docs-sidenav {
    width: 220px;
    margin: 30px 0 0;
    padding: 0;
    background-color: #fff;
    -webkit-border-radius: 6px;
    -moz-border-radius: 6px;
    border-radius: 6px;
    -webkit-box-shadow: 0 1px 4px rgba(0, 0, 0, .065);
    -moz-box-shadow: 0 1px 4px rgba(0, 0, 0, .065);
    box-shadow: 0 1px 4px rgba(0, 0, 0, .065);
}

.bs-docs-sidebar {
    margin-left: 0;
}

.nav-list > li > a {
    padding: 9px 15px;
}

.bs-docs-sidenav > li > a {
    display: block;
    width: 190px \9;
    margin: 0 0 -1px;
    padding: 8px 14px;
    border: 1px solid #e5e5e5;
}

.bs-docs-sidenav > li:first-child > a {
    -webkit-border-radius: 6px 6px 0 0;
    -moz-border-radius: 6px 6px 0 0;
    border-radius: 6px 6px 0 0;
}

.bs-docs-sidenav > li:last-child > a {
    -webkit-border-radius: 0 0 6px 6px;
    -moz-border-radius: 0 0 6px 6px;
    border-radius: 0 0 6px 6px;
}

.bs-docs-sidenav > .active > a {
    position: relative;
    z-index: 2;
    padding: 9px 15px;
    border: 0;
    text-shadow: 0 1px 0 rgba(0, 0, 0, .15);
    -webkit-box-shadow: inset 1px 0 0 rgba(0, 0, 0, .1), inset -1px 0 0 rgba(0, 0, 0, .1);
    -moz-box-shadow: inset 1px 0 0 rgba(0, 0, 0, .1), inset -1px 0 0 rgba(0, 0, 0, .1);
    box-shadow: inset 1px 0 0 rgba(0, 0, 0, .1), inset -1px 0 0 rgba(0, 0, 0, .1);
}

/* Chevrons */
.bs-docs-sidenav .icon-chevron-right {
    float: right;
    margin-top: 2px;
    margin-right: -6px;
    opacity: .25;
}

.bs-docs-sidenav > li > a:hover {
    background-color: #f5f5f5;
}

.bs-docs-sidenav a:hover .icon-chevron-right {
    opacity: .5;
}

.bs-docs-sidenav .active .icon-chevron-right, .bs-docs-sidenav .active a:hover .icon-chevron-right {
    background-image: url(../img/glyphicons-halflings-white.png);
    opacity: 1;
}

.bs-docs-sidenav.affix {
    top: 40px;
}

.bs-docs-sidenav.affix-bottom {
    position: absolute;
    top: auto;
    bottom: 270px;
}

.chart-control {
                 margin-top: 40px;
                 padding-top: 30px;
 }


#metrics-list .activated a,
#filters-list .activated a,
#values-list .activated a {
                            background-color: #666;
                            color: #fff;
}

.hero-unit {
    position: relative;
}
.hero-unit .close {
    font-size: 32px;
}
.hero-unit .chart-controls {
    margin-bottom: 20px;
}

.chart-area svg {
         height: 450px;
 }

.navbar_filters {

}
.navbar_filters .navbar-inner {
    padding-left: 10px;
    padding-right: 10px;
}
.navbar_filters .nav > li > a {
    padding: 5px;
}
.remove-chart {
    position: absolute;
    top: 14px;
    right: 14px;
}
.navbar_filters .delete-bar {
    font-size: 28px
}
.chart-control {
    margin-bottom: 10px;
}
.chart-control .input_date {
    margin-bottom: 0;
}
